=== modified file '.bzrignore'
--- .bzrignore	2011-02-08 00:21:55 +0000
+++ .bzrignore	2011-06-18 22:06:36 +0000
@@ -5,3 +5,4 @@
 .spyderproject
 sloecode/public/help
 twistd.pid
+./docs/build/*

=== modified file 'TODO'
--- TODO	2011-02-13 07:08:42 +0000
+++ TODO	2011-05-23 21:03:21 +0000
@@ -1,15 +1,46 @@
 Stuff that needs to be done in sloecode, in no particular order:
 ----------------------------------------------------------------
 
- * Make it look pretty.
-  - style.css needs to be hacked.
-  - templates need proper CSS classes added to lots of tags.
-  - do we want to support themes? If so, need to add support for them in the
-    python code (simple enough to do).
-
  * Make User & Project creation forms use AJAX to check if the user login or
    project name already exists. It'd be nice to be able to do this with AJAX,
    but we'd still need some sort of error checking on the process_foo controller
    actions that had the ability to re-display the original form with suitable
    error messages.
 
+Database Migration:
+
+We need to support database migration as we change the database over time. I
+don't think there's any point in trying to detect database changes
+automatically. Instead, I suggest the following be done to support database
+migration:
+
+ 1. keep a text file in the project root called DBVERSION. This file contains a
+    single number, which represents the database version number as it is in
+    trunk.
+
+ 2. Every time we change the database schema, we write a simple python file that
+    contains a couple of SQL commands to update the database to the new version.
+    Most of the time the commands will be the same for different database
+    backends, so we can make it really simple - a single SQL string. When the
+    case is more complicated, we may need a string each for every supported
+    backend.
+
+ 3. A script that gets run as part of the upgrade process, which checks the
+    version from DBVERSION, and checks the version from within the database, and
+    automatically applies every script required to update the database to the
+    current version.
+
+    Note that this script needs to be invoked both when the package is updated,
+    and every time the user gets a new version from source control.
+
+Notes:
+------
+
+ * Since we need to test upgrade SQL scripts against different backends, I suggest
+   we select which backends we want to support. At a minimum, we should support
+   MySQL and SQLite.
+
+ * It should be reasonably easy to write a script that grabs a version of the code
+   from bzr, creates the database, then grabs the latest version of the code and
+   runs the upgrade script. We could make this an automated process for the backends
+   we want to support.

=== added file 'database.ini'
--- database.ini	1970-01-01 00:00:00 +0000
+++ database.ini	2011-06-04 07:57:38 +0000
@@ -0,0 +1,11 @@
+; database config file for sloecode. See main sloecode file for other 
+; (non-database) configuration options.
+
+[database]
+dbuser = 
+dbpass = 
+dbname = development.db
+dbserver = 
+dbport = 
+dbtype = sqlite3
+dbpath = 

=== added directory 'dbpatches'
=== added file 'dbpatches/README.txt'
--- dbpatches/README.txt	1970-01-01 00:00:00 +0000
+++ dbpatches/README.txt	2011-06-02 04:37:11 +0000
@@ -0,0 +1,7 @@
+
+Database Schema:
+================
+
+Scripts in this file contain the base schema for the database.
+
+

=== added file 'dbpatches/mysql'
--- dbpatches/mysql	1970-01-01 00:00:00 +0000
+++ dbpatches/mysql	2011-06-04 04:02:19 +0000
@@ -0,0 +1,186 @@
+-- MySQL dump 10.13  Distrib 5.1.54, for debian-linux-gnu (x86_64)
+--
+-- Host: localhost    Database: sloecode
+-- ------------------------------------------------------
+-- Server version	5.1.54-1ubuntu4
+
+/*!40101 SET @OLD_CHARACTER_SET_CLIENT=@@CHARACTER_SET_CLIENT */;
+/*!40101 SET @OLD_CHARACTER_SET_RESULTS=@@CHARACTER_SET_RESULTS */;
+/*!40101 SET @OLD_COLLATION_CONNECTION=@@COLLATION_CONNECTION */;
+/*!40101 SET NAMES utf8 */;
+/*!40103 SET @OLD_TIME_ZONE=@@TIME_ZONE */;
+/*!40103 SET TIME_ZONE='+00:00' */;
+/*!40014 SET @OLD_UNIQUE_CHECKS=@@UNIQUE_CHECKS, UNIQUE_CHECKS=0 */;
+/*!40014 SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0 */;
+/*!40101 SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE='NO_AUTO_VALUE_ON_ZERO' */;
+/*!40111 SET @OLD_SQL_NOTES=@@SQL_NOTES, SQL_NOTES=0 */;
+
+--
+-- Table structure for table `database_version`
+--
+
+DROP TABLE IF EXISTS `database_version`;
+/*!40101 SET @saved_cs_client     = @@character_set_client */;
+/*!40101 SET character_set_client = utf8 */;
+CREATE TABLE `database_version` (
+  `version` int(11) NOT NULL AUTO_INCREMENT,
+  PRIMARY KEY (`version`)
+) ENGINE=MyISAM AUTO_INCREMENT=2 DEFAULT CHARSET=latin1;
+/*!40101 SET character_set_client = @saved_cs_client */;
+
+--
+-- Dumping data for table `database_version`
+--
+
+LOCK TABLES `database_version` WRITE;
+/*!40000 ALTER TABLE `database_version` DISABLE KEYS */;
+INSERT INTO `database_version` VALUES (1);
+/*!40000 ALTER TABLE `database_version` ENABLE KEYS */;
+UNLOCK TABLES;
+
+--
+-- Table structure for table `person`
+--
+
+DROP TABLE IF EXISTS `person`;
+/*!40101 SET @saved_cs_client     = @@character_set_client */;
+/*!40101 SET character_set_client = utf8 */;
+CREATE TABLE `person` (
+  `id` int(11) NOT NULL AUTO_INCREMENT,
+  `login` varchar(200) DEFAULT NULL,
+  `password` varchar(200) DEFAULT NULL,
+  `name` varchar(200) DEFAULT NULL,
+  `email` varchar(200) DEFAULT NULL,
+  PRIMARY KEY (`id`),
+  UNIQUE KEY `login` (`login`)
+) ENGINE=MyISAM AUTO_INCREMENT=2 DEFAULT CHARSET=latin1;
+/*!40101 SET character_set_client = @saved_cs_client */;
+
+--
+-- Dumping data for table `person`
+--
+
+LOCK TABLES `person` WRITE;
+/*!40000 ALTER TABLE `person` DISABLE KEYS */;
+INSERT INTO `person` VALUES (1,'admin','8c6976e5b5410415bde908bd4dee15dfb167a9c873fc4bb8a81f6f2ab448a918','Default Administrator',NULL);
+/*!40000 ALTER TABLE `person` ENABLE KEYS */;
+UNLOCK TABLES;
+
+--
+-- Table structure for table `person_keys`
+--
+
+DROP TABLE IF EXISTS `person_keys`;
+/*!40101 SET @saved_cs_client     = @@character_set_client */;
+/*!40101 SET character_set_client = utf8 */;
+CREATE TABLE `person_keys` (
+  `id` int(11) NOT NULL AUTO_INCREMENT,
+  `person_id` int(11) DEFAULT NULL,
+  `key_name` text,
+  `key_type` text,
+  `key_data` text,
+  `key_comment` text,
+  PRIMARY KEY (`id`),
+  KEY `person_id` (`person_id`)
+) ENGINE=MyISAM DEFAULT CHARSET=latin1;
+/*!40101 SET character_set_client = @saved_cs_client */;
+
+--
+-- Dumping data for table `person_keys`
+--
+
+LOCK TABLES `person_keys` WRITE;
+/*!40000 ALTER TABLE `person_keys` DISABLE KEYS */;
+/*!40000 ALTER TABLE `person_keys` ENABLE KEYS */;
+UNLOCK TABLES;
+
+--
+-- Table structure for table `person_project_membership`
+--
+
+DROP TABLE IF EXISTS `person_project_membership`;
+/*!40101 SET @saved_cs_client     = @@character_set_client */;
+/*!40101 SET character_set_client = utf8 */;
+CREATE TABLE `person_project_membership` (
+  `id` int(11) NOT NULL AUTO_INCREMENT,
+  `person_id` int(11) DEFAULT NULL,
+  `project_id` int(11) DEFAULT NULL,
+  `role` enum('Observer','Developer','Manager') DEFAULT NULL,
+  PRIMARY KEY (`id`),
+  KEY `project_id` (`project_id`),
+  KEY `person_id` (`person_id`)
+) ENGINE=MyISAM DEFAULT CHARSET=latin1;
+/*!40101 SET character_set_client = @saved_cs_client */;
+
+--
+-- Dumping data for table `person_project_membership`
+--
+
+LOCK TABLES `person_project_membership` WRITE;
+/*!40000 ALTER TABLE `person_project_membership` DISABLE KEYS */;
+/*!40000 ALTER TABLE `person_project_membership` ENABLE KEYS */;
+UNLOCK TABLES;
+
+--
+-- Table structure for table `project`
+--
+
+DROP TABLE IF EXISTS `project`;
+/*!40101 SET @saved_cs_client     = @@character_set_client */;
+/*!40101 SET character_set_client = utf8 */;
+CREATE TABLE `project` (
+  `id` int(11) NOT NULL AUTO_INCREMENT,
+  `name` varchar(200) DEFAULT NULL,
+  `displayname` varchar(200) DEFAULT NULL,
+  `description` text,
+  PRIMARY KEY (`id`),
+  UNIQUE KEY `displayname` (`displayname`),
+  UNIQUE KEY `name` (`name`)
+) ENGINE=MyISAM DEFAULT CHARSET=latin1;
+/*!40101 SET character_set_client = @saved_cs_client */;
+
+--
+-- Dumping data for table `project`
+--
+
+LOCK TABLES `project` WRITE;
+/*!40000 ALTER TABLE `project` DISABLE KEYS */;
+/*!40000 ALTER TABLE `project` ENABLE KEYS */;
+UNLOCK TABLES;
+
+--
+-- Table structure for table `site_role`
+--
+
+DROP TABLE IF EXISTS `site_role`;
+/*!40101 SET @saved_cs_client     = @@character_set_client */;
+/*!40101 SET character_set_client = utf8 */;
+CREATE TABLE `site_role` (
+  `id` int(11) NOT NULL AUTO_INCREMENT,
+  `person_id` int(11) DEFAULT NULL,
+  `role` enum('User Admin','Project Admin') DEFAULT NULL,
+  PRIMARY KEY (`id`),
+  KEY `person_id` (`person_id`)
+) ENGINE=MyISAM AUTO_INCREMENT=3 DEFAULT CHARSET=latin1;
+/*!40101 SET character_set_client = @saved_cs_client */;
+
+--
+-- Dumping data for table `site_role`
+--
+
+LOCK TABLES `site_role` WRITE;
+/*!40000 ALTER TABLE `site_role` DISABLE KEYS */;
+INSERT INTO `site_role` VALUES (1,1,'User Admin'),(2,1,'Project Admin');
+/*!40000 ALTER TABLE `site_role` ENABLE KEYS */;
+UNLOCK TABLES;
+/*!40103 SET TIME_ZONE=@OLD_TIME_ZONE */;
+
+/*!40101 SET SQL_MODE=@OLD_SQL_MODE */;
+/*!40014 SET FOREIGN_KEY_CHECKS=@OLD_FOREIGN_KEY_CHECKS */;
+/*!40014 SET UNIQUE_CHECKS=@OLD_UNIQUE_CHECKS */;
+/*!40101 SET CHARACTER_SET_CLIENT=@OLD_CHARACTER_SET_CLIENT */;
+/*!40101 SET CHARACTER_SET_RESULTS=@OLD_CHARACTER_SET_RESULTS */;
+/*!40101 SET COLLATION_CONNECTION=@OLD_COLLATION_CONNECTION */;
+/*!40111 SET SQL_NOTES=@OLD_SQL_NOTES */;
+
+-- Dump completed on 2011-06-04 15:36:54

=== added file 'dbpatches/sqlite3'
--- dbpatches/sqlite3	1970-01-01 00:00:00 +0000
+++ dbpatches/sqlite3	2011-06-02 04:37:11 +0000
@@ -0,0 +1,57 @@
+PRAGMA foreign_keys=OFF;
+BEGIN TRANSACTION;
+CREATE TABLE project (
+	id INTEGER NOT NULL, 
+	name VARCHAR(200), 
+	displayname VARCHAR(200), 
+	description TEXT, 
+	PRIMARY KEY (id), 
+	UNIQUE (displayname), 
+	UNIQUE (name)
+);
+CREATE TABLE database_version (
+	version INTEGER NOT NULL, 
+	PRIMARY KEY (version)
+);
+INSERT INTO "database_version" VALUES(1);
+CREATE TABLE person (
+	id INTEGER NOT NULL, 
+	login VARCHAR(200), 
+	password VARCHAR(200), 
+	name VARCHAR(200), 
+	email VARCHAR(200), 
+	PRIMARY KEY (id), 
+	UNIQUE (login)
+);
+INSERT INTO "person" VALUES(1,'admin','8c6976e5b5410415bde908bd4dee15dfb167a9c873fc4bb8a81f6f2ab448a918','Default Administrator',NULL);
+CREATE TABLE person_project_membership (
+	id INTEGER NOT NULL, 
+	person_id INTEGER, 
+	project_id INTEGER, 
+	role VARCHAR(9), 
+	PRIMARY KEY (id), 
+	CHECK (role IN ('Observer', 'Developer', 'Manager')), 
+	FOREIGN KEY(project_id) REFERENCES project (id), 
+	FOREIGN KEY(person_id) REFERENCES person (id)
+);
+CREATE TABLE site_role (
+	id INTEGER NOT NULL, 
+	person_id INTEGER, 
+	role VARCHAR(13), 
+	PRIMARY KEY (id), 
+	CHECK (role IN ('User Admin', 'Project Admin')), 
+	FOREIGN KEY(person_id) REFERENCES person (id)
+);
+INSERT INTO "site_role" VALUES(1,1,'User Admin');
+INSERT INTO "site_role" VALUES(2,1,'Project Admin');
+CREATE TABLE person_keys (
+	id INTEGER NOT NULL, 
+	person_id INTEGER, 
+	key_name TEXT, 
+	key_type TEXT, 
+	key_data TEXT, 
+	key_comment TEXT, 
+	PRIMARY KEY (id), 
+	FOREIGN KEY(person_id) REFERENCES person (id)
+);
+COMMIT;

=== modified file 'development.ini'
--- development.ini	2011-05-18 07:08:19 +0000
+++ development.ini	2011-06-04 07:57:38 +0000
@@ -44,14 +44,8 @@
 beaker.session.key = sloecode
 beaker.session.secret = somesecret
 
-# If you'd like to fine-tune the individual locations of the cache data dirs
-# for the Cache data, or the Session saves, un-comment the desired settings
-# here:
-#beaker.cache.data_dir = %(here)s/data/cache
-#beaker.session.data_dir = %(here)s/data/sessions
-
-# SQLAlchemy database URL
-sqlalchemy.url = sqlite:///%(here)s/development.db
+# Database configuration is in the following file:
+db_config_file = %(here)s/database.ini
 
 # WARNING: *THE LINE BELOW MUST BE UNCOMMENTED ON A PRODUCTION ENVIRONMENT*
 # Debug mode will enable the interactive debugging tool, allowing ANYONE to

=== removed file 'production.ini'
--- production.ini	2011-05-27 23:00:04 +0000
+++ production.ini	1970-01-01 00:00:00 +0000
@@ -1,104 +0,0 @@
-#
-# configuration of the Pylons application 'sloecode' for production use
-#
-
-# Configure your contact email address and SMTP server
-[DEFAULT]
-debug = true
-email_to = root@localhost
-smtp_server = localhost
-error_email_from = paste@localhost
-
-# Bazaar directory locations are shared across the webapp and the smart server
-# so I'm putting them into the DEFAULT section:
-bzr_root = /srv/sloecode/bzr-repos
-bzr_project_root = %(bzr_root)s/project
-bzr_person_root = %(bzr_root)s/person
-bzr_deleted_project_root = %(bzr_root)s/deleted_project
-bzr_deleted_person_root = %(bzr_root)s/deleted_person
-
-# Configuration for twistd smart server:
-[smartserver]
-access_log_name = /var/log/sloecode/smartserver-access.log
-log_name = /var/log/sloecode/smartserver.log
-private_key_file = /var/sloecode/keys/sloecode_rsa
-public_key_file = /var/sloecode/keys/sloecode_rsa.pub
-ssh_port_number = 22
-ssh_port = tcp:%(ssh_port_number)s
-idle_timeout = 3600
-banner = Sloecode Bazaar Smart Server
-
-xmlrpc_auth_endpoint = http://localhost/xmlrpc
-
-[server:main]
-use = egg:Paste#http
-host = 0.0.0.0
-port = 80
-
-# Configure the Pylons application
-# IMPORTANT: change the session_secret to another long string
-[app:main]
-use = egg:sloecode
-full_stack = true
-static_files = true
-cache_dir = /var/cache/pylons-sloecode
-
-
-beaker.session.key = sloecode
-beaker.session.secret = HJtS52NNQzesc87wuO4zvU8CI90M9OCg
-
-# Specify the database for SQLAlchemy to use:
-
-#sqlalchemy.url = sqlite:///location/of/sqlite.db
-#sqlalchemy.url = postgres://username:password@host/database
-#sqlalchemy.url = mysql://username:password@host/database
-
-# Note that absolute URLs for sqlite need four (4) '/' characters
-# like so:
-sqlalchemy.url = sqlite:////srv/sloecode/database.db
-
-# Only enable debug mode if needed. It's dangerous in production use!
-set debug = false
-
-# Logging configuration
-[loggers]
-keys = root, routes, sloecode, sqlalchemy
-
-[handlers]
-keys = console
-
-[formatters]
-keys = generic
-
-[logger_root]
-level = INFO
-handlers = console
-
-[logger_routes]
-level = DEBUG
-handlers =
-qualname = routes.middleware
-# "level = DEBUG" logs the route matched and routing variables.
-
-[logger_sloecode]
-level = DEBUG
-handlers =
-qualname = sloecode
-
-[logger_sqlalchemy]
-level = INFO
-handlers =
-qualname = sqlalchemy.engine
-# "level = INFO" logs SQL queries.
-# "level = DEBUG" logs SQL queries and results.
-# "level = WARN" logs neither.  (Recommended for production systems.)
-
-[handler_console]
-class = StreamHandler
-args = (sys.stderr,)
-level = NOTSET
-formatter = generic
-
-[formatter_generic]
-format = %(asctime)s,%(msecs)03d %(levelname)-5.5s [%(name)s] [%(threadName)s] %(message)s
-datefmt = %H:%M:%S

=== added file 'production.ini'
--- production.ini	1970-01-01 00:00:00 +0000
+++ production.ini	2011-06-04 05:35:16 +0000
@@ -0,0 +1,97 @@
+#
+# configuration of the Pylons application 'sloecode' for production use
+#
+
+# Configure your contact email address and SMTP server
+[DEFAULT]
+debug = true
+email_to = root@localhost
+smtp_server = localhost
+error_email_from = paste@localhost
+
+# Bazaar directory locations are shared across the webapp and the smart server
+# so I'm putting them into the DEFAULT section:
+bzr_root = /srv/sloecode/bzr-repos
+bzr_project_root = %(bzr_root)s/project
+bzr_person_root = %(bzr_root)s/person
+bzr_deleted_project_root = %(bzr_root)s/deleted_project
+bzr_deleted_person_root = %(bzr_root)s/deleted_person
+
+# Configuration for twistd smart server:
+[smartserver]
+access_log_name = /var/log/sloecode/smartserver-access.log
+log_name = /var/log/sloecode/smartserver.log
+private_key_file = /var/sloecode/keys/sloecode_rsa
+public_key_file = /var/sloecode/keys/sloecode_rsa.pub
+ssh_port_number = 22
+ssh_port = tcp:%(ssh_port_number)s
+idle_timeout = 3600
+banner = Sloecode Bazaar Smart Server
+
+xmlrpc_auth_endpoint = http://localhost/xmlrpc
+
+[server:main]
+use = egg:Paste#http
+host = 0.0.0.0
+port = 80
+
+# Configure the Pylons application
+# IMPORTANT: change the session_secret to another long string
+[app:main]
+use = egg:sloecode
+full_stack = true
+static_files = true
+cache_dir = /var/cache/pylons-sloecode
+
+
+beaker.session.key = sloecode
+beaker.session.secret = HJtS52NNQzesc87wuO4zvU8CI90M9OCg
+
+# Database configuration is in the following file:
+db_config_file = %(here)s/database.ini
+
+# Only enable debug mode if needed. It's dangerous in production use!
+set debug = false
+
+# Logging configuration
+[loggers]
+keys = root, routes, sloecode, sqlalchemy
+
+[handlers]
+keys = console
+
+[formatters]
+keys = generic
+
+[logger_root]
+level = INFO
+handlers = console
+
+[logger_routes]
+level = DEBUG
+handlers =
+qualname = routes.middleware
+# "level = DEBUG" logs the route matched and routing variables.
+
+[logger_sloecode]
+level = DEBUG
+handlers =
+qualname = sloecode
+
+[logger_sqlalchemy]
+level = INFO
+handlers =
+qualname = sqlalchemy.engine
+# "level = INFO" logs SQL queries.
+# "level = DEBUG" logs SQL queries and results.
+# "level = WARN" logs neither.  (Recommended for production systems.)
+
+[handler_console]
+class = StreamHandler
+args = (sys.stderr,)
+level = NOTSET
+formatter = generic
+
+[formatter_generic]
+format = %(asctime)s,%(msecs)03d %(levelname)-5.5s [%(name)s] [%(threadName)s] %(message)s
+datefmt = %H:%M:%S

=== modified file 'sloecode/bzr/factory.py'
--- sloecode/bzr/factory.py	2011-06-03 20:55:20 +0000
+++ sloecode/bzr/factory.py	2011-06-04 08:19:28 +0000
@@ -11,7 +11,7 @@
 
 def ensure_directory_exists(dir_name):
     """Make a directory if it doesn't already exist.
-    
+
     Throws RuntimeError if the path already exists as something other
     than a directory (a file, for example).
     """
@@ -37,13 +37,13 @@
 class BazaarRepositoryFactory(object):
     """Responsible for creating repositories for people and projects."""
 
-    def __init__(self, person_root_dir, project_root_dir, 
+    def __init__(self, person_root_dir, project_root_dir,
     deleted_person_dir, deleted_project_dir):
         self.person_root_dir = person_root_dir
         self.project_root_dir = project_root_dir
         self.deleted_person_dir = deleted_person_dir
         self.deleted_project_dir = deleted_project_dir
-        
+
         ensure_directory_exists(self.person_root_dir)
         ensure_directory_exists(self.project_root_dir)
         ensure_directory_exists(self.deleted_person_dir)
@@ -54,17 +54,20 @@
         Get the directory for a users repository, given the username.
         """
         return os.path.join(self.person_root_dir, person_name)
-    
+
     def get_project_repo_dir(self, project_name):
         """
         Get the directory for a project repository, given the project name.
         """
         return os.path.join(self.project_root_dir, project_name)
-    
+
     def create_shared_repository_for_person(self, person_name):
         """Create an empty shared repo for person."""
         dir_name = self.get_user_repo_dir(person_name)
-        create_shared_repo(dir_name)
+        if os.path.exists(dir_name):
+            print "Warning: path '%s' already exists, not creating repository for user '%s'" % (dir_name, person_name)
+        else:
+            create_shared_repo(dir_name)
 
     def create_shared_repository_for_project(self, project_name):
         """Create a repository for the project with a trunk branch."""
@@ -74,53 +77,53 @@
         to_transport = repo.user_transport.clone('trunk')
         format = bzrdir.format_registry.make_bzrdir('default')
         branch = bzrdir.BzrDir.create_branch_convenience(
-            to_transport.base, format=format, 
+            to_transport.base, format=format,
             possible_transports=[to_transport])
         branch.set_append_revisions_only(True)
-        
+
     def get_repository_for_project(self, project_name):
         """Get a bzrlib.repository.Repository object for a given project
         name.
         """
         dir_name = self.get_project_repo_dir(project_name)
         return BazaarRepositoryAdaptor(Repository.open(dir_name))
-        
+
     def get_repository_for_person(self, person_name):
         """
         Get a bzrlib.repository.Repository object for a given person login.
         """
         dir_name = self.get_user_repo_dir(person_name)
         return BazaarRepositoryAdaptor(Repository.open(dir_name))
-    
+
     def delete_shared_repository_for_person(self, person_name):
         """
-        Delete a personal repository. This function should only be called 
+        Delete a personal repository. This function should only be called
         AFTER the user account has been deleted. This function will create a
         copy of the users repository in the personal bzr repo backup folder.
         """
         src_dir = self.get_user_repo_dir(person_name)
         dst_dir = self.get_delete_folder_for_user(person_name)
-        
+
         copytree(src_dir, dst_dir)
         rmtree(src_dir)
-        
+
     def delete_shared_repository_for_project(self, project_name):
         """
-        Delete a project repository. This function should only be called 
+        Delete a project repository. This function should only be called
         AFTER the project has been deleted. This function will create a
         copy of the project repository in the project repo backup folder.
         """
         src_dir = self.get_project_repo_dir(project_name)
         dst_dir = self.get_delete_folder_for_project(project_name)
-        
+
         copytree(src_dir, dst_dir)
         rmtree(src_dir)
-        
+
     def get_delete_folder_for_user(self, person_name):
         """
         Get a path to a directory that does not yet exist, suitable for backing
         up a user repository.
-        
+
         Directories are in the form <username>-<timestamp>, allowing
         administrators to easily differentiate between multiple user backups over
         time.
@@ -134,14 +137,14 @@
                 dirpath = dirpath[:-2]
             dirpath += '_' + str(attempt)
             attempt += 1
-            
+
         return dirpath
-    
+
     def get_delete_folder_for_project(self, project_name):
         """
         Get a path to a directory that does not yet exist, suitable for backing
         up a user repository.
-        
+
         Directories are in the form <username>-<timestamp>, allowing
         administrators to easily differentiate between multiple user backups over
         time.
@@ -155,5 +158,4 @@
                 dirpath = dirpath[:-2]
             dirpath += '_' + str(attempt)
             attempt += 1
-            
         return dirpath

=== modified file 'sloecode/config/environment.py'
--- sloecode/config/environment.py	2011-05-18 07:08:19 +0000
+++ sloecode/config/environment.py	2011-06-04 06:41:58 +0000
@@ -1,10 +1,12 @@
 """Pylons environment configuration"""
 import os
 
+from ConfigParser import RawConfigParser
 from jinja2 import ChoiceLoader, Environment, FileSystemLoader
+import os.path
 import pylons
 from pylons.configuration import PylonsConfig
-from sqlalchemy import engine_from_config
+from sqlalchemy import create_engine
 
 from sloecode.bzr.factory import BazaarRepositoryFactory
 import sloecode.lib.app_globals as app_globals
@@ -45,16 +47,59 @@
     config['pylons.strict_tmpl_context'] = True
 
     # Setup the SQLAlchemy database engine
-    engine = engine_from_config(config, 'sqlalchemy.')
+    # do this by creating the engine url from components:
+    db_config = RawConfigParser()
+    db_config.read(config['db_config_file'])
+    
+    dialect = db_config.get('database', 'dbtype')
+    # translate from dbconfig-common terminology
+    if dialect == 'sqlite3':
+        dialect = 'sqlite'
+    username = db_config.get('database', 'dbuser')
+    password = db_config.get('database', 'dbpass')
+    schema_name = db_config.get('database', 'dbname')
+    db_server = db_config.get('database', 'dbserver')
+    db_port = db_config.get('database', 'dbport')
+    db_path = db_config.get('database', 'dbpath')
+    
+    # we support sqlite:// and mysql://
+    engine_url = dialect + '://'
+    # sqlite needs 4 '/''s, and needs special path options:
+    if dialect == 'sqlite':
+        engine_url += '/' + os.path.join(db_path, schema_name)
+    else:
+        engine_url += "%s:%s@%s/%s" % (username, password, db_server, schema_name)
+    
+    engine = create_engine(engine_url)
     init_model(engine)
 
-    # CONFIGURATION OPTIONS HERE (note: all config options will override
-    # any Pylons config options)
+    # create the Bazaar factory:
     config['bzr.factory'] = BazaarRepositoryFactory(
         global_conf['bzr_person_root'],
         global_conf['bzr_project_root'],
         global_conf['bzr_deleted_person_root'],
         global_conf['bzr_deleted_project_root'],
         )
+        
+    # This is a bit of a nasty hack: In "developer mode", we explicitly call:
+    #
+    # `paster setup-app /path/to/config.ini`
+    #
+    # Which creates all the database tables, and also created the admin user and
+    # creates the repository for the admin user - this works fine.
+    #
+    # However, in release mode, the database schema is created by dbconfig-common
+    # and upgraded from one version to the next by the same. We never get a chance
+    # to create the Bazaar repository for the admin user, which results in crashes.
+    #
+    # The following code checks to see if the repository has been created for the
+    # admin user, and creates it if it hasn't:
+    from bzrlib.errors import NotBranchError
+
+    try:
+        repo = config['bzr.factory'].get_repository_for_person('admin')
+    except NotBranchError:
+        config['bzr.factory'].create_shared_repository_for_person('admin')
+
 
     return config

=== modified file 'sloecode/lib/readconf.py'
--- sloecode/lib/readconf.py	2011-02-15 09:14:09 +0000
+++ sloecode/lib/readconf.py	2011-05-27 02:43:41 +0000
@@ -3,12 +3,17 @@
 from ConfigParser import ConfigParser
 
 
-def get_conf(section_name):
-    """Read the config file."""
-    try:
-        file_path = os.environ['SLOECODE_CONFIG']
-    except KeyError:
-        raise RuntimeError('Missing SLOECODE_CONFIG environment')
+def get_conf(section_name, file_path=None):
+    """
+    Read a section from the config file. If no file path is given, it will be
+    read from the system environment variable SLOECODE_CONFIG.
+    """
+    if file_path is None:
+        try:
+            file_path = os.environ['SLOECODE_CONFIG']
+        except KeyError:
+            raise RuntimeError('Missing SLOECODE_CONFIG environment')
+
     cp = ConfigParser()
     cp.read(file_path)
     abs_file_path = os.path.abspath(file_path)

=== modified file 'sloecode/websetup.py'
--- sloecode/websetup.py	2011-05-18 07:08:19 +0000
+++ sloecode/websetup.py	2011-06-08 07:49:31 +0000
@@ -34,7 +34,7 @@
 
     Session.commit()
     log.info("Successfully setup")
-    
+
     # Ensure that the directories exist for the repositories.
     fact = BazaarRepositoryFactory(
         conf.global_conf['bzr_person_root'],
@@ -43,8 +43,8 @@
         conf.global_conf['bzr_deleted_project_root'],
         )
     # create repo for admin user we created above:
-    # TODO: this will crash if the admin user was already created, which 
-    # will be the case if the database has been wiped or moved from one 
-    # backend to another. We should detect this and warn about it, but 
+    # TODO: this will crash if the admin user was already created, which
+    # will be the case if the database has been wiped or moved from one
+    # backend to another. We should detect this and warn about it, but
     # probably not crash!
     fact.create_shared_repository_for_person(admin.login)

=== added file 'upgrade_database.py'
--- upgrade_database.py	1970-01-01 00:00:00 +0000
+++ upgrade_database.py	2011-05-29 00:28:02 +0000
@@ -0,0 +1,99 @@
+#!/usr/bin/env python
+
+"""
+Script to upgrade a live database schema.
+"""
+
+from sys import argv
+if len(argv) != 2:
+    print "Usage: upgrade_database.py /path/to/sloecode/config.ini"
+    exit(1)
+
+from sqlalchemy import engine_from_config
+from sloecode.lib.readconf import get_conf
+import os
+import re
+
+def main():
+    patch_dir = 'dbpatches'
+    config = get_conf('app:main', argv[1])
+    Engine = engine_from_config(config)
+    Connection = Engine.connect()
+
+    # we now have the connection and engine configured. First, get the current
+    # database version number from the database:
+    dbversion = 0
+    try:
+        result = Connection.execute(r"SELECT MAX(version) from database_version;").fetchone()
+        dbversion = result[0]
+    except:
+        pass
+    
+    print "Database version is currently:", dbversion
+
+    # Get the current new database version from trunk - this is our stop point:
+    enddbversion = int(open('DBVERSION','r').read())
+    print "Upgrading database to version:", enddbversion
+
+    # anything to do?
+    if enddbversion <= dbversion:
+        print "Nothing to do!"
+        exit(0)
+
+    print "Database type is:", Engine.name
+    backend = Engine.name
+
+    patch_re = re.compile(r'^sloecode-(\d+)(-%s)?.sql' % (backend))
+
+    # store all patches in dictionary according to patch number:
+    patches = {}
+
+    # find all patch files in dbpatches folder.
+    # TODO: Ensure we're looking at the correct folder. How will this work with
+    # debian package upgrades? We'll need to store these in the filesystem somewhere!
+    for patch_file in os.listdir(patch_dir):
+        match = patch_re.match(patch_file)
+        if match:
+            groups = match.groups()
+            patch_number = int(groups[0])
+            # not that we care, but we can tell if the patch is backend specific here.
+            # backend-generic patches will have None in 'groups[1]'.
+            patches[patch_number] = patch_file
+
+    # make sure we have all the patch files we need. Bail if we're missing anything:
+    for i in range(dbversion+1, enddbversion+1):
+        if i not in patches:
+            print "Error: Cannot find patch to upgrade schema from version %d to %d." % (i-1, i)
+            exit(1)
+
+    # loop through every patch number, and apply patch WITHIN A DATABASE TRANSACTION!
+    trans = Connection.begin()
+    try:
+        for i in range(dbversion+1, enddbversion+1):
+            apply_patch(Connection, os.path.join(patch_dir, patches[i]), i)
+        trans.commit()
+    except:
+        trans.rollback()
+        raise
+    #
+    # If the patch fails, roll back the transaction. Perhaps this should be a nested
+    # transaction, so we can roll back the entire operation?
+    #
+    # Also, we need to know what kind of back-end we're dealing with. For any
+    # particular version number, the patch will either be sloecode-NN.sql or
+    # sloecode-NN-mysql|sqlite.sql
+
+def apply_patch(conn, patch_file, version):
+    trans = conn.begin()
+    try:
+        print "Applying patch:", patch_file
+        conn.execute(open(patch_file).read())
+        # update the DB version:
+        conn.execute('INSERT INTO database_version (version) VALUES (%d)' % version)
+        trans.commit()
+    except:
+        trans.rollback()
+        raise
+
+if __name__ == '__main__':
+    main()

